@@ -1,6 +1,7 @@
 #if defined(__clang__) && !defined(__OBJC_RUNTIME_INTERNAL__)
 #pragma clang system_header
 #endif
+#include "objc-visibility.h"
 
 #ifndef __LIBOBJC_RUNTIME_H_INCLUDED__
 #define __LIBOBJC_RUNTIME_H_INCLUDED__
@@ -25,6 +26,12 @@ extern "C" {
 #	endif
 #endif
 
+#if defined(__OBJC__) && __has_attribute(ns_returns_retained)
+#	define OBJC_RETURNS_RETAINED __attribute__((ns_returns_retained))
+#else
+#	define OBJC_RETURNS_RETAINED
+#endif
+
 // Make sure we get the limit macros, even in C++ mode
 #ifndef __STDC_LIMIT_MACROS                                                     
 #	define __STDC_LIMIT_MACROS 1
@@ -124,7 +131,7 @@ typedef struct objc_method *Method;
 #	ifdef STRICT_APPLE_COMPATIBILITY
 typedef signed char BOOL;
 #	else
-#		ifdef __vxwords
+#		if defined(__vxworks) || defined(_WIN32)
 typedef  int BOOL;
 #		else
 typedef unsigned char BOOL;
@@ -189,26 +196,32 @@ typedef struct
 
 
 #ifndef YES
-#	define YES ((BOOL)1)
+#	if __has_feature(objc_bool)
+#		define YES __objc_yes
+#	else
+#		define YES ((BOOL)1)
+#	endif
 #endif
 #ifndef NO
-#	define NO ((BOOL)0)
+#	if __has_feature(objc_bool)
+#		define NO __objc_no
+#	else
+#		define NO ((BOOL)0)
+#	endif
 #endif
 
-#ifdef __GNUC
-#	define _OBJC_NULL_PTR __null
-#elif defined(__cplusplus)
-#	define _OBJC_NULL_PTR 0
+#if __has_feature(cxx_nullptr)
+#	define _OBJC_NULL_PTR nullptr
 #else
-#	define _OBJC_NULL_PTR ((void*)0)
+#	define _OBJC_NULL_PTR NULL
 #endif
 
 #ifndef nil
-#	define nil ((id)_OBJC_NULL_PTR)
+#	define nil _OBJC_NULL_PTR
 #endif
 
 #ifndef Nil
-#	define Nil ((Class)_OBJC_NULL_PTR)
+#	define Nil _OBJC_NULL_PTR
 #endif
 
 #include "slot.h"
@@ -220,6 +233,7 @@ typedef struct
  * registered by the runtime.  The alignment must be the base-2 logarithm of
  * the alignment requirement and the types should be an Objective-C type encoding.
  */
+OBJC_PUBLIC
 BOOL class_addIvar(Class cls,
                    const char *name,
                    size_t size,
@@ -229,11 +243,13 @@ BOOL class_addIvar(Class cls,
 /**
  * Adds a method to the class.
  */
+OBJC_PUBLIC
 BOOL class_addMethod(Class cls, SEL name, IMP imp, const char *types);
 
 /**
  * Adds a protocol to the class.
  */
+OBJC_PUBLIC
 BOOL class_addProtocol(Class cls, Protocol *protocol);
 
 /**
@@ -241,6 +257,7 @@ BOOL class_addProtocol(Class cls, Protocol *protocol);
  * name are regarded as equivalent, even if they have different methods.  This
  * behaviour will change in a future version.
  */
+OBJC_PUBLIC
 BOOL class_conformsToProtocol(Class cls, Protocol *protocol);
 
 /**
@@ -248,6 +265,7 @@ BOOL class_conformsToProtocol(Class cls, Protocol *protocol);
  * the outCount argument is set to the number of instance variables returned.
  * The caller is responsible for freeing the returned buffer.
  */
+OBJC_PUBLIC
 Ivar* class_copyIvarList(Class cls, unsigned int *outCount);
 
 /**
@@ -255,6 +273,7 @@ Ivar* class_copyIvarList(Class cls, unsigned int *outCount);
  * outCount argument is set to the number of methods returned.  The caller is
  * responsible for freeing the returned buffer.
  */
+OBJC_PUBLIC
 Method * class_copyMethodList(Class cls, unsigned int *outCount);
 
 /**
@@ -262,6 +281,7 @@ Method * class_copyMethodList(Class cls, unsigned int *outCount);
  * the outCount argument is set to the number of declared properties returned.
  * The caller is responsible for freeing the returned buffer.
  */
+OBJC_PUBLIC
 objc_property_t* class_copyPropertyList(Class cls, unsigned int *outCount);
 
 /**
@@ -269,11 +289,13 @@ objc_property_t* class_copyPropertyList(Class cls, unsigned int *outCount);
  * outCount argument is set to the number of protocols returned.  The caller is
  * responsible for freeing the returned buffer.
  */
+OBJC_PUBLIC
 Protocol *__unsafe_unretained* class_copyProtocolList(Class cls, unsigned int *outCount);
 
 /**
  * Creates an instance of this class, allocating memory using malloc.
  */
+OBJC_PUBLIC OBJC_RETURNS_RETAINED
 id class_createInstance(Class cls, size_t extraBytes);
 
 /**
@@ -281,6 +303,7 @@ id class_createInstance(Class cls, size_t extraBytes);
  * class.  This is an opaque data type and must be accessed with the method_*()
  * family of functions.
  */
+OBJC_PUBLIC
 Method class_getClassMethod(Class aClass, SEL aSelector);
 
 /**
@@ -288,6 +311,7 @@ Method class_getClassMethod(Class aClass, SEL aSelector);
  * this class.  This is an opaque data type and must be accessed with the
  * ivar_*() family of functions.
  */
+OBJC_PUBLIC
 Ivar class_getClassVariable(Class cls, const char* name);
 
 /**
@@ -295,6 +319,7 @@ Ivar class_getClassVariable(Class cls, const char* name);
  * in this class.  This is an opaque data type and must be accessed with the
  * method_*() family of functions.
  */
+OBJC_PUBLIC
 Method class_getInstanceMethod(Class aClass, SEL aSelector);
 
 /**
@@ -302,6 +327,7 @@ Method class_getInstanceMethod(Class aClass, SEL aSelector);
  * class's superclasses must be loaded before this call, or the result is
  * undefined with the non-fragile ABI.
  */
+OBJC_PUBLIC
 size_t class_getInstanceSize(Class cls);
 
 /**
@@ -309,22 +335,26 @@ size_t class_getInstanceSize(Class cls);
  * returning a pointer to the instance variable definition or a null
  * pointer if no instance variable of that name was found.
  */
+OBJC_PUBLIC
 Ivar class_getInstanceVariable(Class cls, const char* name);
 
 /**
  * Sets the object value of a specified instance variable.
  */
+OBJC_PUBLIC
 void object_setIvar(id object, Ivar ivar, id value);
 /**
  * Sets a named instance variable to the value specified by *value.  Note that
  * the instance variable must be a pointer-sized quantity.
  */
+OBJC_PUBLIC
 Ivar object_setInstanceVariable(id obj, const char *name, void *value);
 
 /**
  * Returns the value of the named instance variable.  This should not be used
  * with instance variables that are not pointers.
  */
+OBJC_PUBLIC
 id object_getIvar(id object, Ivar ivar);
 
 /**
@@ -333,6 +363,7 @@ id object_getIvar(id object, Ivar ivar);
  *
  * Note that the instance variable must be a pointer-sized quantity.
  */
+OBJC_PUBLIC
 Ivar object_getInstanceVariable(id obj, const char *name, void **outValue);
 
 /**
@@ -340,54 +371,62 @@ Ivar object_getInstanceVariable(id obj, const char *name, void **outValue);
  * the receiver does not have a method corresponding to this message then this
  * function may return a runtime function that performs forwarding.
  */
+OBJC_PUBLIC
 IMP class_getMethodImplementation(Class cls, SEL name);
 
 /**
  * Identical to class_getMethodImplementation().
  */
+OBJC_PUBLIC
 IMP class_getMethodImplementation_stret(Class cls, SEL name);
 
 /**
  * Returns the name of the class.  This string is owned by the runtime and is
  * valid for (at least) as long as the class remains loaded.
  */
+OBJC_PUBLIC
 const char * class_getName(Class cls);
 
 /**
  * Retrieves metadata about the property with the specified name.
  */
+OBJC_PUBLIC
 objc_property_t class_getProperty(Class cls, const char *name);
 
 /**
  * Returns the superclass of the specified class.
  */
+OBJC_PUBLIC
 Class class_getSuperclass(Class cls);
 
 /**
  * Returns the version of the class.  Currently, the class version is not used
  * inside the runtime at all, however it may be used for the developer-mode ABI.
  */
+OBJC_PUBLIC
 int class_getVersion(Class theClass);
 
 /**
  * Sets the version for this class.
  */
+OBJC_PUBLIC
 void class_setVersion(Class theClass, int version);
 
-OBJC_GNUSTEP_RUNTIME_UNSUPPORTED("Weak instance variables")
+OBJC_PUBLIC OBJC_GNUSTEP_RUNTIME_UNSUPPORTED("Weak instance variables")
 const char *class_getWeakIvarLayout(Class cls);
 
 /**
  * Returns whether the class is a metaclass.  This can be used in conjunction
  * with object_getClass() for differentiating between objects and classes.
  */
+OBJC_PUBLIC
 BOOL class_isMetaClass(Class cls);
 
 /**
  * Registers an alias for the class. Returns YES if the alias could be
  * registered successfully.
  */
-OBJC_NONPORTABLE
+OBJC_PUBLIC OBJC_NONPORTABLE
 BOOL class_registerAlias_np(Class cls, const char *alias);
 
 /**
@@ -395,6 +434,7 @@ BOOL class_registerAlias_np(Class cls, const char *alias);
  * Objective-C runtime uses typed selectors, however the types of the selector
  * will be ignored and a new selector registered with the specified types.
  */
+OBJC_PUBLIC
 IMP class_replaceMethod(Class cls, SEL name, IMP imp, const char *types);
 
 /**
@@ -403,17 +443,20 @@ IMP class_replaceMethod(Class cls, SEL name, IMP imp, const char *types);
  * or more of the various forwarding mechanisms, then this will still return
  * NO.
  */
+OBJC_PUBLIC
 BOOL class_respondsToSelector(Class cls, SEL sel);
 
 /**
  * Returns the instance variable layout of this class as an opaque list that
  * can be applied to other classes.
  */
+OBJC_PUBLIC
 const char *class_getIvarLayout(Class cls);
 /**
  * Sets the class's instance variable layout.  The layout argument must be a
  * value returned by class_getIvarLayout().
  */
+OBJC_PUBLIC
 void class_setIvarLayout(Class cls, const char *layout);
 
 /**
@@ -421,26 +464,29 @@ void class_setIvarLayout(Class cls, const char *layout);
  * because modifying the superclass of a class at run time is a very complex
  * operation and this function is almost always used incorrectly.
  */
-__attribute__((deprecated))
+OBJC_PUBLIC __attribute__((deprecated))
 Class class_setSuperclass(Class cls, Class newSuper);
 
-OBJC_GNUSTEP_RUNTIME_UNSUPPORTED("Weak instance variables")
+OBJC_PUBLIC OBJC_GNUSTEP_RUNTIME_UNSUPPORTED("Weak instance variables")
 void class_setWeakIvarLayout(Class cls, const char *layout);
 
 /**
  * Returns the name of an instance variable.
  */
+OBJC_PUBLIC
 const char* ivar_getName(Ivar ivar);
 
 /**
  * Returns the offset of an instance variable.  This value can be added to the
  * object pointer to get the address of the instance variable.
  */
+OBJC_PUBLIC
 ptrdiff_t ivar_getOffset(Ivar ivar);
 
 /**
  * Returns the Objective-C type encoding of the instance variable.
  */
+OBJC_PUBLIC
 const char* ivar_getTypeEncoding(Ivar ivar);
 
 /**
@@ -449,12 +495,14 @@ const char* ivar_getTypeEncoding(Ivar ivar);
  * Objective-C method will be the self and _cmd parameters, so the returned
  * value will be "@" and ":" respectively.
  */
+OBJC_PUBLIC
 char* method_copyArgumentType(Method method, unsigned int index);
 
 /**
  * Copies the type encoding of an argument of this method.  The caller is
  * responsible for freeing the returned C string.
  */
+OBJC_PUBLIC
 char* method_copyReturnType(Method method);
 
 /**
@@ -462,6 +510,7 @@ char* method_copyReturnType(Method method);
  * expensive on the GNUstep runtime and its use is discouraged.  It is
  * recommended that users call class_replaceMethod() instead.
  */
+OBJC_PUBLIC
 void method_exchangeImplementations(Method m1, Method m2);
 
 /**
@@ -471,11 +520,13 @@ void method_exchangeImplementations(Method m1, Method m2);
  * output string if there is enough space for the argument type and the NULL
  * terminator.  Its use is therefore discouraged.
  */
+OBJC_PUBLIC
 void method_getArgumentType(Method method, unsigned int index, char *dst, size_t dst_len);
 
 /**
  * Returns a pointer to the function used to implement this method.
  */
+OBJC_PUBLIC
 IMP method_getImplementation(Method method);
 
 /**
@@ -485,12 +536,14 @@ IMP method_getImplementation(Method method);
  * although calling method_getTypeEncoding() is faster if you just require the
  * types.
  */
+OBJC_PUBLIC
 SEL method_getName(Method method);
 
 /**
  * Returns the number of arguments (including self and _cmd) that this method
  * expects.
  */
+OBJC_PUBLIC
 unsigned method_getNumberOfArguments(Method method);
 
 /**
@@ -500,6 +553,7 @@ unsigned method_getNumberOfArguments(Method method);
  * output string if there is enough space for the argument type and the NULL
  * terminator.  Its use is therefore discouraged.
  */
+OBJC_PUBLIC
 void method_getReturnType(Method method, char *dst, size_t dst_len);
 
 /**
@@ -507,6 +561,7 @@ void method_getReturnType(Method method, char *dst, size_t dst_len);
  * runtime and will persist for (at least) as long as the class owning the
  * method is loaded.
  */
+OBJC_PUBLIC
 const char * method_getTypeEncoding(Method method);
 
 /**
@@ -514,6 +569,7 @@ const char * method_getTypeEncoding(Method method);
  * expensive with the GNUstep runtime and its use is discouraged.  It is
  * recommended that you call class_replaceMethod() instead.
  */
+OBJC_PUBLIC
 IMP method_setImplementation(Method method, IMP imp);
 
 /**
@@ -522,6 +578,7 @@ IMP method_setImplementation(Method method, IMP imp);
  * used for class variables by adding instance variables to the returned
  * metaclass.
  */
+OBJC_PUBLIC
 Class objc_allocateClassPair(Class superclass, const char *name, size_t extraBytes);
 
 /**
@@ -529,6 +586,7 @@ Class objc_allocateClassPair(Class superclass, const char *name, size_t extraByt
  * attempts to send messages to instances of this class or its subclasses
  * result in undefined behaviour.
  */
+OBJC_PUBLIC
 void objc_disposeClassPair(Class cls);
 
 /**
@@ -537,6 +595,7 @@ void objc_disposeClassPair(Class cls);
  * loaded, it calls the _objc_lookup_class() callback to allow an external
  * library to load the module providing this class.
  */
+OBJC_PUBLIC
 id objc_getClass(const char *name);
 
 /**
@@ -545,18 +604,21 @@ id objc_getClass(const char *name);
  * is 0, it returns the total number of classes registered with the runtime.
  * Otherwise, it copies classes and returns the number copied.
  */
+OBJC_PUBLIC
 int objc_getClassList(Class *buffer, int bufferLen);
 /**
  * Returns a copy of the list of all classes in the system.  The caller is
  * responsible for freeing this list.  The number of classes is returned in the
  * parameter.
  */
+OBJC_PUBLIC
 Class *objc_copyClassList(unsigned int *outCount);
 
 /**
  * Returns the metaclass with the specified name.  This is equivalent to
  * calling object_getClass() on the result of objc_getClass().
  */
+OBJC_PUBLIC
 id objc_getMetaClass(const char *name);
 
 /**
@@ -564,17 +626,20 @@ id objc_getMetaClass(const char *name);
  * function should generally only be called early on in a program, to ensure
  * that all required libraries are loaded.
  */
+OBJC_PUBLIC
 id objc_getRequiredClass(const char *name);
 
 /**
  * Looks up the class with the specified name, but does not invoke any
  * external lazy loading mechanisms.
  */
+OBJC_PUBLIC
 id objc_lookUpClass(const char *name);
 
 /**
  * Returns the protocol with the specified name.
  */
+OBJC_PUBLIC
 Protocol *objc_getProtocol(const char *name);
 /**
  * Allocates a new protocol.  This returns NULL if a protocol with the same
@@ -583,15 +648,18 @@ Protocol *objc_getProtocol(const char *name);
  * Protocols are immutable after they have been registered, so may only be
  * modified between calling this function and calling objc_registerProtocol().
  */
+OBJC_PUBLIC
 Protocol *objc_allocateProtocol(const char *name);
 /**
  * Registers a protocol with the runtime.  After this point, the protocol may
  * not be modified.
  */
+OBJC_PUBLIC
 void objc_registerProtocol(Protocol *proto);
 /**
  * Adds a method to the protocol.
  */
+OBJC_PUBLIC
 void protocol_addMethodDescription(Protocol *aProtocol,
                                    SEL name,
                                    const char *types,
@@ -600,10 +668,12 @@ void protocol_addMethodDescription(Protocol *aProtocol,
 /**
  * Adds a protocol to the protocol.
  */
+OBJC_PUBLIC
 void protocol_addProtocol(Protocol *aProtocol, Protocol *addition);
 /**
  * Adds a property to the protocol.
  */
+OBJC_PUBLIC
 void protocol_addProperty(Protocol *aProtocol,
                           const char *name,
                           const objc_property_attribute_t *attributes,
@@ -618,6 +688,7 @@ void protocol_addProperty(Protocol *aProtocol,
  * adding instance variables and methods to it.  A class can not have instance
  * variables added to it after objc_registerClassPair() has been called.
  */
+OBJC_PUBLIC
 void objc_registerClassPair(Class cls);
 
 /**
@@ -625,6 +696,7 @@ void objc_registerClassPair(Class cls);
  * object.  This is a pointer to the storage specified with the extraBytes
  * parameter given when allocating an object.
  */
+OBJC_PUBLIC
 void *object_getIndexedIvars(id obj);
 
 // FIXME: The GNU runtime has a version of this which omits the size parameter
@@ -633,30 +705,35 @@ void *object_getIndexedIvars(id obj);
 /**
  * Free an object created with class_createInstance().
  */
+OBJC_PUBLIC
 id object_dispose(id obj);
 
 /**
  * Returns the class of the object.  Note: the isa pointer should not be
  * accessed directly with the GNUstep runtime.
  */
+OBJC_PUBLIC
 Class object_getClass(id obj);
 
 /**
  * Sets the class of the object.  Note: the isa pointer should not be
  * accessed directly with the GNUstep runtime.
  */
+OBJC_PUBLIC
 Class object_setClass(id obj, Class cls);
 
 /**
  * Returns the name of the class of the object.  This is equivalent to calling
  * class_getName() on the result of object_getClass().
  */
+OBJC_PUBLIC
 const char *object_getClassName(id obj);
 
 
 /**
  * Returns the name of a specified property.
  */
+OBJC_PUBLIC
 const char *property_getName(objc_property_t property);
 
 /**
@@ -665,11 +742,13 @@ const char *property_getName(objc_property_t property);
  * description of the format for this string may be found in Apple's
  * Objective-C Runtime Programming Guide.
  */
+OBJC_PUBLIC
 const char *property_getAttributes(objc_property_t property);
 
 /**
  * Returns an array of attributes for this property.
  */
+OBJC_PUBLIC
 objc_property_attribute_t *property_copyAttributeList(objc_property_t property,
                                                       unsigned int *outCount);
 /**
@@ -677,6 +756,7 @@ objc_property_attribute_t *property_copyAttributeList(objc_property_t property,
  * that this only sets the property metadata.  The property accessor methods
  * must already be created.
  */
+OBJC_PUBLIC
 BOOL class_addProperty(Class cls,
                        const char *name,
                        const objc_property_attribute_t *attributes, 
@@ -686,6 +766,7 @@ BOOL class_addProperty(Class cls,
  * Replaces property metadata.  If the property does not exist, then this is
  * equivalent to calling class_addProperty().
  */
+OBJC_PUBLIC
 void class_replaceProperty(Class cls,
                            const char *name,
                            const objc_property_attribute_t *attributes,
@@ -694,12 +775,14 @@ void class_replaceProperty(Class cls,
 /**
  * Returns a copy of a single attribute.
  */
+OBJC_PUBLIC
 char *property_copyAttributeValue(objc_property_t property,
                                   const char *attributeName);
 
 /**
  * Testswhether a protocol conforms to another protocol.
  */
+OBJC_PUBLIC
 BOOL protocol_conformsToProtocol(Protocol *p, Protocol *other);
 
 /**
@@ -707,21 +790,33 @@ BOOL protocol_conformsToProtocol(Protocol *p, Protocol *other);
  * the array in the variable pointed to by the last parameter.  The caller is
  * responsible for freeing this array.
  */
+OBJC_PUBLIC
 struct objc_method_description *protocol_copyMethodDescriptionList(Protocol *p,
 	BOOL isRequiredMethod, BOOL isInstanceMethod, unsigned int *count);
 
 /**
- * Returns an array of property metadata values, with the number being stored
- * in the variable pointed to by the last argument.  The caller is responsible
- * for freeing the returned array.
+ * Returns an array of required instance properties, with the number being
+ * stored in the variable pointed to by the last argument.  The caller is
+ * responsible for freeing the returned array.
  */
+OBJC_PUBLIC
 objc_property_t *protocol_copyPropertyList(Protocol *p, unsigned int *count);
 
+/**
+ * Returns an array of properties specified by this class, with the number
+ * being stored in the variable pointed to by the last argument.  The caller is
+ * responsible for freeing the returned array.
+ */
+OBJC_PUBLIC
+objc_property_t *protocol_copyPropertyList2(Protocol *p, unsigned int *count,
+	BOOL isRequiredProperty, BOOL isInstanceProperty);
+
 /**
  * Returns an array of protocols that this protocol conforms to, with the
  * number of protocols in the array being returned via the last argument.  The
  * caller is responsible for freeing this array.
  */
+OBJC_PUBLIC
 Protocol *__unsafe_unretained*protocol_copyProtocolList(Protocol *p, unsigned int *count);
 
 /**
@@ -732,26 +827,37 @@ Protocol *__unsafe_unretained*protocol_copyProtocolList(Protocol *p, unsigned in
  *
  * The caller is responsible for freeing the returned array.
  */
+OBJC_PUBLIC
 Protocol *__unsafe_unretained*objc_copyProtocolList(unsigned int *outCount);
 /**
  * Returns the method description for the specified method within a given
  * protocol.
  */
+OBJC_PUBLIC
 struct objc_method_description protocol_getMethodDescription(Protocol *p,
 	SEL aSel, BOOL isRequiredMethod, BOOL isInstanceMethod);
 
+/**
+ * Returns the extended type encoding of the specified method.
+ * 
+ * Note: This function is used by JavaScriptCore but is not public in Apple's
+ * implementation and so its semantics may change in the future and this
+ * runtime may diverge from Apple's.
+ */
+OBJC_PUBLIC
+const char *_protocol_getMethodTypeEncoding(Protocol *p, SEL aSel,
+	BOOL isRequiredMethod, BOOL isInstanceMethod);
+
 /**
  * Returns the name of the specified protocol.
  */
+OBJC_PUBLIC
 const char* protocol_getName(Protocol *p);
 
 /**
  * Returns the property metadata for the property with the specified name.
- *
- * Note: The Apple documentation for this method contains some nonsense for
- * isInstanceProperty.  As there is no language syntax for defining properties
- * on classes, we return NULL if this is not YES.
  */
+OBJC_PUBLIC
 objc_property_t protocol_getProperty(Protocol *p, const char *name,
 	BOOL isRequiredProperty, BOOL isInstanceProperty);
 
@@ -761,6 +867,7 @@ objc_property_t protocol_getProperty(Protocol *p, const char *name,
  * which made not attempt to unique protocols (or even register them with the
  * runtime).
  */
+OBJC_PUBLIC
 BOOL protocol_isEqual(Protocol *p, Protocol *other);
 
 /**
@@ -768,21 +875,25 @@ BOOL protocol_isEqual(Protocol *p, Protocol *other);
  * the function (either a method or a forwarding hook) that should be called in
  * response to a given message.
  */
+OBJC_PUBLIC
 IMP objc_msg_lookup(id, SEL) OBJC_NONPORTABLE;
 /**
  * The message lookup function used for messages sent to super in the GCC ABI.
  * This specifies both the class and the 
  */
+OBJC_PUBLIC
 IMP objc_msg_lookup_super(struct objc_super*, SEL) OBJC_NONPORTABLE;
 
 /**
  * Returns the name of the specified selector.
  */
+OBJC_PUBLIC
 const char *sel_getName(SEL sel);
 
 /**
  * Registers a selector with the runtime.  This is equivalent to sel_registerName().
  */
+OBJC_PUBLIC
 SEL sel_getUid(const char *selName);
 
 /**
@@ -795,22 +906,26 @@ SEL sel_getUid(const char *selName);
  * both b and c are typed selectors with different types, then then the first
  * two will return YES, but the third case will return NO.
  */
+OBJC_PUBLIC
 BOOL sel_isEqual(SEL sel1, SEL sel2);
 
 /**
  * Registers an untyped selector with the runtime.
  */
+OBJC_PUBLIC
 SEL sel_registerName(const char *selName);
 
 /**
  * Register a typed selector.
  */
+OBJC_PUBLIC
 SEL sel_registerTypedName_np(const char *selName, const char *types) OBJC_NONPORTABLE;
 
 /**
  * Returns the type encoding associated with a selector, or the empty string is
  * there is no such type.
  */
+OBJC_PUBLIC
 const char *sel_getType_np(SEL aSel) OBJC_NONPORTABLE;
 
 /**
@@ -821,6 +936,7 @@ const char *sel_getType_np(SEL aSel) OBJC_NONPORTABLE;
  * once with a relatively small on-stack buffer and then only call it again
  * with a heap-allocated buffer if there is not enough space.
  */
+OBJC_PUBLIC
 unsigned sel_copyTypes_np(const char *selName, const char **types, unsigned count) OBJC_NONPORTABLE;
 
 /**
@@ -831,20 +947,57 @@ unsigned sel_copyTypes_np(const char *selName, const char **types, unsigned coun
  * once with a relatively small on-stack buffer and then only call it again
  * with a heap-allocated buffer if there is not enough space.
  */
+OBJC_PUBLIC
 unsigned sel_copyTypedSelectors_np(const char *selName, SEL *const sels, unsigned count) OBJC_NONPORTABLE;
 
 /**
  * New ABI lookup function.  Receiver may be modified during lookup or proxy
  * forwarding and the sender may affect how lookup occurs.
  */
+OBJC_PUBLIC
 extern struct objc_slot *objc_msg_lookup_sender(id *receiver, SEL selector, id sender)
+	OBJC_NONPORTABLE OBJC_DEPRECATED;
+
+/**
+ * Deprecated function for accessing a slot without going via any forwarding
+ * mechanisms.
+ */
+OBJC_PUBLIC
+extern struct objc_slot *objc_get_slot(Class, SEL)
+	OBJC_NONPORTABLE OBJC_DEPRECATED;
+
+/**
+ * Look up a slot, without invoking any forwarding mechanisms.  The third
+ * parameter is used to return a pointer to the current value of the version
+ * counter.  If this value is equal to `objc_method_cache_version` then the
+ * slot is safe to reuse without performing another lookup.
+ */
+OBJC_PUBLIC
+extern struct objc_slot2 *objc_get_slot2(Class, SEL, uint64_t*)
 	OBJC_NONPORTABLE;
 
+/**
+ * Look up a slot, invoking any required forwarding mechanisms.  The third
+ * parameter is used to return a pointer to the current value of the version
+ * counter.  If this value is equal to `objc_method_cache_version` then the
+ * slot is safe to reuse without performing another lookup.
+ */
+OBJC_PUBLIC
+extern struct objc_slot2 *objc_slot_lookup_version(id *receiver, SEL selector, uint64_t*)
+	OBJC_NONPORTABLE;
+
+/**
+ * Look up a slot, invoking any required forwarding mechanisms.
+ */
+OBJC_PUBLIC
+extern IMP objc_msg_lookup2(id *receiver, SEL selector) OBJC_NONPORTABLE;
+
 /**
  * Registers a class for small objects.  Small objects are stored inside a
  * pointer.  If the class can be registered, then this returns YES.  The second
  * argument specifies the bit pattern to use to identify the small object.
  */
+OBJC_PUBLIC
 BOOL objc_registerSmallObjectClass_np(Class cls, uintptr_t classId);
 
 /**
@@ -908,7 +1061,8 @@ typedef uintptr_t objc_AssociationPolicy;
  * Returns an object previously stored by calling objc_setAssociatedObject()
  * with the same arguments, or nil if none exists.
  */
-id objc_getAssociatedObject(id object, void *key);
+OBJC_PUBLIC
+id objc_getAssociatedObject(id object, const void *key);
 /**
  * Associates an object with another.  This provides a mechanism for storing
  * extra state with an object, beyond its declared instance variables.  The
@@ -917,10 +1071,12 @@ id objc_getAssociatedObject(id object, void *key);
  * value may be any object, but must respond to -copy or -retain, and -release,
  * if an association policy of copy or retain is passed as the final argument.
  */
-void objc_setAssociatedObject(id object, void *key, id value, objc_AssociationPolicy policy);
+OBJC_PUBLIC
+void objc_setAssociatedObject(id object, const void *key, id value, objc_AssociationPolicy policy);
 /**
  * Removes all associations from an object.  
  */
+OBJC_PUBLIC
 void objc_removeAssociatedObjects(id object);
 
 /**
@@ -928,49 +1084,57 @@ void objc_removeAssociatedObjects(id object);
  * take an object pointer (self) as its first argument, and then the same
  * arguments as the method.
  */
-IMP imp_implementationWithBlock(void *block);
+OBJC_PUBLIC
+IMP imp_implementationWithBlock(id block);
 /**
  * Returns the type encoding of an IMP that would be returned by passing the
  * block to imp_implementationWithBlock().  Returns NULL if this is not a valid
  * block encoding for transforming to an IMP (it must take id as its first
  * argument).  The caller is responsible for freeing the returned value.
  */
-char *block_copyIMPTypeEncoding_np(void*block);
+OBJC_PUBLIC
+char *block_copyIMPTypeEncoding_np(id block);
 /**
  * Returns the block that was used in an IMP created by
  * imp_implementationWithBlock().  The result of calling this function with any
  * other IMP is undefined.
  */
-void *imp_getBlock(IMP anImp);
+OBJC_PUBLIC
+id imp_getBlock(IMP anImp);
 /**
  * Removes a block that was converted to an IMP with
  * imp_implementationWithBlock().  The result of calling this function with any
  * other IMP is undefined.  Returns YES on success, NO on failure.
  */
+OBJC_PUBLIC
 BOOL imp_removeBlock(IMP anImp);
 
 /**
  * Adds a method to a specific object,  This method will not be added to any
  * other instances of the same class.
  */
+OBJC_PUBLIC
 BOOL object_addMethod_np(id object, SEL name, IMP imp, const char *types);
 
 /**
  * Replaces a method on a specific object,  This method will not be added to
  * any other instances of the same class.
  */
+OBJC_PUBLIC
 IMP object_replaceMethod_np(id object, SEL name, IMP imp, const char *types);
 
 /**
  * Creates a clone, in the JavaScript sense - an object which inherits both
  * associated references and methods from the original object.
  */
+OBJC_PUBLIC
 id object_clone_np(id object);
 
 /**
  * Returns the prototype of the object if it was created with
  * object_clone_np(), or nil otherwise.
  */
+OBJC_PUBLIC
 id object_getPrototype_np(id object);
 
 /**
@@ -991,8 +1155,25 @@ id object_getPrototype_np(id object);
  * This currently sets a global value.  In the future, it may be configurable
  * on a per-thread basis.
  */
+OBJC_PUBLIC
 int objc_set_apple_compatible_objcxx_exceptions(int newValue) OBJC_NONPORTABLE;
 
+/** 
+ * This function is inserted by the compiler when a mutation is detected during
+ * a foreach iteration. It is exported as a weak symbol to enable GNUstep or
+ * some other framework to replace it trivially.
+ */
+OBJC_PUBLIC
+void __attribute__((weak)) objc_enumerationMutation(id obj);
+
+/**
+ * Ensure that `+initialize` has been sent to the class of the argument (or the
+ * argument, if it is a class).  This will not call `+initialize` if it has
+ * been called already, either via an explicit call to this function or by
+ * being sent some other message.
+ */
+OBJC_PUBLIC
+void objc_send_initialize(id object) OBJC_NONPORTABLE;
 
 #define _C_ID       '@'
 #define _C_CLASS    '#'
